@@ -3,31 +3,13 @@
 # 2.0, and the MIT License.  See the LICENSE file in the root of this
 # repository for complete details.
 
-import codecs
-import os
-import re
+from importlib import metadata
 
 
-here = os.path.abspath(os.path.dirname(__file__))
-
 # We want an image in the README and include the README in the docs.
 suppress_warnings = ["image.nonlocal_uri"]
 
 
-def read(*parts):
-    return codecs.open(os.path.join(here, *parts), "r").read()
-
-
-def find_version(*file_paths):
-    version_file = read(*file_paths)
-    version_match = re.search(
-        r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M
-    )
-    if version_match:
-        return version_match.group(1)
-    raise RuntimeError("Unable to find version string.")
-
-
 # -- General configuration ----------------------------------------------------
 
 extensions = [
@@ -57,11 +39,12 @@ copyright = f"2013, { author }"
 # The version info for the project you're documenting, acts as replacement for
 # |version| and |release|, also used in various other places throughout the
 # built documents.
-#
-# The short X.Y version.
-version = find_version("..", "src", "structlog", "__init__.py")
+
 # The full version, including alpha/beta/rc tags.
-release = ""
+release = metadata.version("structlog")
+# The short X.Y version.
+version = release.rsplit(".", 1)[0]
+
 exclude_patterns = ["_build"]
 
 # The reST default role (used for this markup: `text`) to use for all
@@ -90,6 +73,10 @@ add_function_parentheses = True
 # unit titles (such as .. function::).
 # add_module_names = True
 
+# Move type hints into the description block, instead of the func definition.
+autodoc_typehints = "description"
+autodoc_typehints_description_target = "documented"
+
 # -- Options for HTML output --------------------------------------------------
 
 # The theme to use for HTML and HTML Help pages.  See the documentation for
