@@ -1,3 +1,4 @@
+# SPDX-License-Identifier: MIT OR Apache-2.0
 # This file is dual licensed under the terms of the Apache License, Version
 # 2.0, and the MIT License.  See the LICENSE file in the root of this
 # repository for complete details.
@@ -208,6 +209,37 @@ class TestConsoleRenderer:
             + styles.reset
         ) == rv
 
+    def test_key_values_unsorted(self, styles, padded):
+        """
+        Key-value pairs go in original order to the end.
+        """
+        cr = dev.ConsoleRenderer(sort_keys=False)
+
+        rv = cr(
+            None,
+            None,
+            {"event": "test", "key": "value", "foo": "bar"},
+        )
+
+        assert (
+            padded
+            + styles.kv_key
+            + "key"
+            + styles.reset
+            + "="
+            + styles.kv_value
+            + "value"
+            + styles.reset
+            + " "
+            + styles.kv_key
+            + "foo"
+            + styles.reset
+            + "="
+            + styles.kv_value
+            + "bar"
+            + styles.reset
+        ) == rv
+
     @pytest.mark.parametrize("wrap", [True, False])
     def test_exception_rendered(self, cr, padded, recwarn, wrap):
         """
